@@ -10,6 +10,18 @@ test_that("snpchip throws an error w/ invalid chr coord", {
   expect_error(snpchip(c("ch13:32446842", "chr13:32447222", "rs148890987"), "ALL", token = Sys.getenv("LDLINK_TOKEN")))
 })
 
+test_that("snpchip throws an error w/ invalid genome_build", {
+  skip_on_cran()
+  expect_error(SNPchip(snps = c("rs3", "rs148890987"),
+                       chip = "ALL",
+                       token = Sys.getenv("LDLINK_TOKEN"),
+                       genome_build = "grch999"))
+  expect_error(SNPchip(snps = c("rs3", "rs148890987"),
+                       chip = "ALL",
+                       token = Sys.getenv("LDLINK_TOKEN"),
+                       genome_build = c("grch37", "grch38")))
+})
+
 test_that("snpchip works", {
   skip_on_cran()
   expect_named(SNPchip(c("rs3", "rs4", "rs148890987"), "ALL", token = Sys.getenv("LDLINK_TOKEN")))
@@ -17,5 +29,5 @@ test_that("snpchip works", {
 
 test_that("snpchip works with ALL_Illumina", {
   skip_on_cran()
-  expect_named(SNPchip(c("rs3", "rs4", "rs148890987"), "ALL_Illumina", token = Sys.getenv("LDLINK_TOKEN")))
+  expect_named(SNPchip("rs12082302", "ALL_Illumina", token = Sys.getenv("LDLINK_TOKEN")))
 })
